==> mkreadmes-1.1.2.tar.bz2: mkreadmes-1.1.2/main.c <==
/*
 *	Program: mkreadmes
 *
 *	A much faster version of "make_readmes", the Perl-based FreeBSD ports tool
 *	used in FreeBSD's ports make system for (re)building the ports README.html
 *	files
 *
 *	Unlike the Perl version, this program is designed to be used as a
 *	standalone tool, not intended for integration into the FreeBSD ports
 *	collection's system of Makefiles (although it very well could be)
 *
 *	Depends on the following files within the ports tree:
 *
 *	the ports index file (INDEX-${OSRELEASE}, e.g., INDEX-10), used to gather
 *	information about a specific port
 *
 *	the top-level and category-level Makefiles (to obtain information about
 *	the existing categories and ports)
 *
 *	The default template files (README.top, README.category and README.port),
 *	used as a "blueprint" for generating the README.html files) are installed
 *	under ${PREFIX}/share/mkreadmes/Templates directory.
 *
 *	This file contains only the main() function and a few small auxiliary
 *	functions called directly by main(), plus a few global variables.
 *
 *	The three real "workhorse" routines that drive the program --
 *	make_top_readme(), make_category_readme() and make_port_readme(), as well
 *	as the "wrapper" routine make_readme() which is called from main() --
 *	all are located in the mkreadmes.c file.
 *
 *	$Id: main.c,v 1.47 2012/03/07 16:44:33 conrads Exp $
 *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/param.h>
#include <sys/stat.h>
#include <unistd.h>

#include "mkreadmes.h"

char	ports_dir[PATH_MAX];		/* top-level ports directory */
char	ports_dir_plus[PATH_MAX];	/* ports_dir plus a trailing slash */
char	*category = NULL;			/* pointer to current category */

/* flags set by command line options */
int		normal_paths = 0;			/* make relative paths relative to current
										working directory instead of ports_dir
									 */
int		quiet = 0;					/* silence normal output */
int		verbose = 0;				/* increase the amount of output */

/* prototypes for routines in other source files */

extern	char   *resolve_path(char *path);							/* init.c */
extern	char   *skip_leading(const char *str, const char *pattern);	/* init.c */
extern	int		initialize(void);									/* init.c */
extern	int		make_readme(const char *path, int make_level); /* mkreadmes.c */
extern	int		set_ports_dir(char *path);							/* init.c */
extern	int		set_templates_dir(char *path);						/* init.c */

/******************************************************************************/

static int get_make_level(char *path)
/*
	Distinguish whether the fully qualified 'path' is a port, category or
	top-level directory

	Returns TOP, CATEGORY or PORT if successful, or -1 on error
*/
{
	/* Is it the top-level ports directory (as in "an identical match")? */

	if (strcmp(path, ports_dir) == 0)
	{
		return TOP;
	}

	/* Not top.  OK, then, strip off the leading ports_dir/ and
		see what we've got
	 */

	/* set 'category' to point to the category/port portion of path

		(setting 'category' now saves us a wee bit of work later in
		make_readme_category() and make_readme_port())
	 */

	category = skip_leading(path, ports_dir_plus);

	if (category == path)	/* path doesn't start with ports_dir/ */
	{
		fprintf(stderr, "\"%s\" is not inside the ports tree\n", path);
		return -1;
	}

	/* if the string pointed to by category still contains a slash, then it's
		not at category level, so try it as a port (may be a totally bogus path,
		but let make_readme_port() deal with it at the point where it actually
		fails)
	 */
	if (strcspn(category, "/") != strlen(category))
	{
		return PORT;
	}

	return CATEGORY;	/* let make_readme_category() determine whether it's a
							valid category or not
						 */
}

/******************************************************************************/

static void usage(void)
{
	char usage_text[] =
"\n"
"mkreadmes -- Make README.html files for the FreeBSD Ports Collection\n"
"\n"
"Usage: mkreadmes [options] [pathname-list]\n"
"\n"
"Options:\n"
"-h             Display this help.\n"
"-n             Use normal handling of relative paths, instead of making them\n"
"               relative to ${PORTSDIR} (paths beginning with \".\" or \"..\" are\n"
"               always treated normally).\n"
"-p pathname    Use pathname as the top level of the ports tree.  Any relative\n"
"               paths in pathname-list will be relative to this, unless the \"-n\"\n"
"               switch is used.  Defaults to the setting of the environment\n"
"               variable \"${PORTSDIR}\", if defined, or \"/usr/ports\" otherwise.\n"
"-q             Quiet mode.  Don't output anything except error messages.\n"
"-t pathname    Use the template files stored under pathname instead of the\n"
/* port Makefile will edit the prefix in the next line if needed */
"               default /usr/local/share/mkreadmes/Templates.\n"
"-V             Display program version.\n"
"-v             Verbose mode.  Show more details about the program's progress.\n"
"\n"
"pathname-list is an optional whitespace-separated list of port and category\n"
"pathnames.  If none is provided, default is to make all of the README.html\n"
"files for the entire ports tree (top-level, categories and ports).\n"
"\n"
"\0";

	char *ptr = usage_text;

	while (*ptr)
	{
		printf("%s", ptr);
		ptr += strlen(ptr) + 1;
	}

	exit (0);
}

/******************************************************************************/

int main(int argc, char *argv[])
/*
	And so it begins...

	process command line options, then process each command line pathname
	argument (if any) in turn, or the entire ports tree if none are given

	returns the number of command line arguments which resulted in some
	type of error, or -1 if initialization failed
 */
{
	char *path = NULL;	/* current pathname command line argument */

	char c;				/* character returned from getopt() */

	int make_level;		/* indicator of the level within the ports tree
							that we're starting from for a given command line
							argument; will be set to one of TOP, CATEGORY or
							PORT for each pathname we process
						 */
	int	return_code = 0;	/* code returned by main on exit*/

	extern int optind;	/* index into the argv[] array */

#define suggest_help \
fprintf(stderr, "Try \"mkreadmes -h\" or \"man mkreadmes\" for more information.\n")

	/* process command line options and their arguments */

	while ((c = getopt(argc, argv, ":hnp:qt:Vv")) != -1)
	{
		switch (c)
		{
			case 'h':
				usage();	/* usage() exits after displaying help */
			case 'n':
				++normal_paths;
				break;
			case 'p':
				if (set_ports_dir(optarg) != 0)
					++return_code;
				break;
			case 'q':
				if (verbose)
				{
					fprintf(stderr, "option \"-q\" is incompatible with \"-v\"\n");
					++return_code;
				}
				else ++quiet;	/* Shhh!  Don't say anything!  */
				break;
			case 't':
				if (set_templates_dir(optarg) != 0)
					++return_code;
				break;
			case 'V':
				printf("mkreadmes version ");
				printf(VERSION);
				printf("\n");
				exit (0);
			case 'v':
				if (quiet)
				{
					fprintf(stderr, "option \"-v\" is incompatible with \"-q\"\n");
					++return_code;
				}
				else
				{
					fprintf(stderr, "Increasing verbosity level\n");
					++verbose;
				}
				break;
			case ':':
				fprintf(stderr, "Missing pathname argument to option \"-%c\"\n",
						optopt);
				suggest_help;
				++return_code;
				break;
			case '?':
				fprintf(stderr, "Unknown option: -%c\n", optopt);
				suggest_help;
				++return_code;
				break;
			default:
				fprintf(stderr, "?? getopt returned character code %d ??\n", c);
		}
	}

	if (return_code)		/* errors in command line processing */
		return return_code;

	/* initialize buffers and other data we'll be working with */
	if (initialize() == -1)
	{
		return -1;			/* initialization failed */
	}

	/* if no pathnames given as command line arguments,
		use the default top-level ports directory as set
		by the initialize() routine
	 */
	if (optind == argc)
	{
		return make_readme(ports_dir, TOP);
	}

	/* otherwise, process each command line argument in turn */

	while (optind < argc)
	{
		if ((path = resolve_path(argv[optind++])) == NULL)
		{
			++return_code;	/* note the error and move on to next argument */
			continue;
		}

		if ((make_level = get_make_level(path)) == -1)
		{
			++return_code;	/* ditto */
			continue;
		}

		return_code += make_readme(path, make_level);

		/* free the pointer malloc'ed by realpath() earlier in resolve_path() */
		free(path);
	}

	return return_code;
}

==> mkreadmes-1.2.tar.bz2: mkreadmes-1.2/main.c <==
/*
 *	Program: mkreadmes
 *
 *	A much faster version of "make_readmes", the Perl-based FreeBSD ports tool
 *	used in FreeBSD's ports make system for (re)building the ports README.html
 *	files
 *
 *	Unlike the Perl version, this program is designed to be used as a
 *	standalone tool, not intended for integration into the FreeBSD ports
 *	collection's system of Makefiles (although it very well could be)
 *
 *	Depends on the following files within the ports tree:
 *
 *	the ports index file (INDEX-${OSRELEASE}, e.g., INDEX-10), used to gather
 *	information about a specific port
 *
 *	the top-level and category-level Makefiles (to obtain information about
 *	the existing categories and ports)
 *
 *	The default template files (README.top, README.category and README.port),
 *	used as a "blueprint" for generating the README.html files) are installed
 *	under ${PREFIX}/share/mkreadmes/Templates directory.
 *
 *	This file contains only the main() function and a few small auxiliary
 *	functions called directly by main(), plus a few global variables.
 *
 *	The three real "workhorse" routines that drive the program --
 *	make_top_readme(), make_category_readme() and make_port_readme(), as well
 *	as the "wrapper" routine make_readme() which is called from main() --
 *	all are located in the mkreadmes.c file.
 *
 *	$Id: main.c,v 1.64 2012/03/10 02:20:56 conrads Exp $
 *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/param.h>

#include "mkreadmes.h"

char	ports_dir[PATH_MAX];		/* top-level ports directory */
char	ports_dir_plus[PATH_MAX];	/* ports_dir plus a trailing slash */

extern	char   *category;			/* pointer to name of current category */

/* flags set by command line options */
int		normal_paths = 0;			/* make relative paths relative to current
										working directory instead of ports_dir
									 */
int		quiet = 0;					/* silence normal output */
int		verbose = 0;				/* increase the amount of output */

/* prototypes for routines in other source files */

extern	char   *resolve_path(char *path);							/* util.c */
extern	char   *skip_leading(const char *str, const char *pattern);	/* util.c */
extern	int		initialize(void);									/* init.c */
extern	int		make_readme(const char *path, int make_level); /* mkreadmes.c */
extern	int		set_ports_dir(char *path);							/* init.c */
extern	int		set_templates_dir(char *path);						/* init.c */

/******************************************************************************/

static void usage(void)
/*
	Display the help screen, then exit
 */
{
	/* I originally had usage_text[] declared as one giant string made up of
		the concatenation of the following strings (i.e, the same layout as
		below, but without the commas after each string), but the compiler
		insisted on issuing a warning about the size of the string being larger
		than what the standard requires an implementation to support, so I
		redid it this way to make the warnings go away.  The pointer stuff is a
		little more complicated than before (a single printf did the whole job),
		but it works just fine, and I hate compiler warnings uglying up my
		compiles.  :-)
	 */

	const char *usage_text[] = {
"\n",
"mkreadmes -- Make README.html files for the FreeBSD Ports Collection\n",
"\n",
"Usage: mkreadmes [options] [pathname-list]\n",
"\n",
"Options:\n",
"-h             Display this help.\n",
"-n             Use normal handling of relative paths, instead of making them\n",
"               relative to ${PORTSDIR} (paths beginning with \".\" or \"..\"\n",
"               are always treated normally).\n",
"-p pathname    Use pathname as the top level of the ports tree.  Any relative\n",
"               paths in pathname-list will be relative to this, unless \"-n\"\n",
"               is used.  Defaults to \"/usr/ports\", or the setting of the\n",
"               ${PORTSDIR} environment variable, if defined.\n",
"-q             Quiet mode.  Don't display anything except error messages.\n",
"-t pathname    Use the template files stored under pathname instead of the\n",
/* port Makefile will edit the prefix in the next line if needed */
"               default /usr/local/share/mkreadmes/Templates.\n",
"-V             Display program version.\n",
"-v             Increase verbosity.  May be repeated to display progressively\n",
"               more detailed information about the program's progress.\n",
"\n",
"pathname-list is an optional whitespace-separated list of port and category\n",
"pathnames.  If none is provided, the default is to make all of the README.html\n",
"files for the entire ports tree (top-level, categories and ports).\n",
"\n",
NULL
};

	const char **ptr = &usage_text[0];

	/* loop through the usage_text[] array, printing one line at a time */
	
	while (*ptr)	/* (while ptr is not pointing at a NULL pointer) */
	{
		printf("%s", *ptr++);	/* ptr is a pointer to *pointer* to char, so
									incrementing it bumps us to the next
									pointer in the array, and dereferencing it
									yield's the next string's adress (yeah, it
									confused me, too, at first)  :-)
								 */
	}

	exit (0);
}

/******************************************************************************/

static int get_make_level(char *path)
/*
	Distinguish whether the fully qualified 'path' is a port, category or
	top-level directory

	Returns TOP, CATEGORY or PORT if successful, or -1 on error
*/
{
	/* Is it the top-level ports directory (as in "an identical match")? */

	if (strcmp(path, ports_dir) == 0)
	{
		return TOP;
	}

	/* Not top.  OK, then, strip off the leading ports_dir/ and
		see what we've got
	 */

	/* set 'category' to point to the category/port portion of path

		(setting 'category' now saves us a wee bit of work later in
		make_readme_category() and make_readme_port())
	 */

	category = skip_leading(path, ports_dir_plus);

	if (category == path)	/* path doesn't start with ports_dir/ */
	{
		fprintf(stderr, "\"%s\" is not inside the ports tree\n", path);
		return -1;
	}

	/* if the string pointed to by category still contains a slash, then it's
		not at category level, so try it as a port (may be a totally bogus path,
		but let make_readme_port() deal with it at the point where it actually
		fails)
	 */
	if (strcspn(category, "/") != strlen(category))
	{
		return PORT;
	}

	return CATEGORY;	/* let make_readme_category() determine whether it's a
							valid category or not
						 */
}

/******************************************************************************/

int main(int argc, char *argv[])
/*
	And so it begins...

	process command line options, then process each command line pathname
	argument (if any) in turn, or the entire ports tree if none are given

	returns the number of command line arguments which resulted in some
	type of error, or -1 if initialization failed
 */
{
	char *path = NULL;	/* current pathname command line argument */

	int c;				/* character returned from getopt() */

	int make_level;		/* indicator of the level within the ports tree
							that we're starting from for a given command line
							argument; will be set to one of TOP, CATEGORY or
							PORT for each pathname we process
						 */
	int	return_code = 0;	/* code returned by main on exit*/

	/* process command line options and their arguments */

	while ((c = getopt(argc, argv, ":hnp:qt:Vv")) != -1)
	{
		switch (c)
		{
			case 'h':
				usage();	/* usage() exits after displaying help */
			case 'n':
				/* don't output message more than once */
				if (verbose && !normal_paths)
					fprintf(stderr,
							"Using normal handling of relative paths\n");
				++normal_paths;
				break;
			case 'p':
				if (set_ports_dir(optarg) != 0)
					++return_code;
				break;
			case 'q':
				if (verbose)
				{
					fprintf(stderr,
							"option \"-q\" is incompatible with \"-v\"\n");
					++return_code;
				}
				else ++quiet;	/* Shhh!  Don't say anything! */
				break;
			case 't':
				if (set_templates_dir(optarg) != 0)
					++return_code;
				break;
			case 'V':
				printf("mkreadmes version " VERSION "\n");
				exit (0);
			case 'v':
				if (quiet)
				{
					fprintf(stderr, "option \"-v\" is incompatible with \"-q\"\n");
					++return_code;
				}
				else
				{
					/* don't output message more than once */
					if (!verbose)
						fprintf(stderr, "Increasing verbosity level\n");
					++verbose;
				}
				break;
			case ':':
				fprintf(stderr, "Missing pathname argument to option \"-%c\"\n",
						optopt);
				suggest_help;
				++return_code;
				break;
			case '?':
				fprintf(stderr, "Unknown option: -%c\n", optopt);
				suggest_help;
				++return_code;
				break;
			default:	/* something majorly wrong if we wind up here! */
				fprintf(stderr, "?? getopt returned character code %d ??\n", c);
		}
	}

	if (return_code)		/* errors in command line processing */
		return return_code;

	/* initialize most of the data we'll be needing */
	if (initialize() == -1)
	{
		return -1;			/* initialization failed */
	}

	/* if no pathnames given as command line arguments,
		use the default top-level ports directory as set
		by the initialize() routine (either PORTSDIR or
		"/usr/ports")
	 */
	if (optind == argc)
	{
		return make_readme(ports_dir, TOP);
	}

	/* otherwise, process each command line argument in turn */

	while (optind < argc)
	{
		if ((path = resolve_path(argv[optind++])) == NULL)
		{
			++return_code;	/* note the error and move on to next argument */
			continue;
		}

		if ((make_level = get_make_level(path)) == -1)
		{
			++return_code;	/* ditto */
			continue;
		}

		return_code += make_readme(path, make_level);

		/* free the pointer malloc'ed by realpath() earlier in resolve_path() */
		free(path);
	}

	return return_code;
}
